import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'app_responsive.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatelessWidget {
  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return AppProvider(
      child: MaterialApp(
        title: 'Flutter Demo',
        theme: ThemeData(
          primarySwatch: Colors.blue,
          visualDensity: VisualDensity.adaptivePlatformDensity,
        ),
        home: ExamplePage(),
      ),
    );
  }
}

class ExamplePage extends IPage {
  @override
  IState<IPage, IController> createState() => ExampleState();
}

class ExampleState extends IState<ExamplePage, ExampleController> {
  final ExampleController _controller = ExampleController();

  @override
  Widget buildChild(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("app responsive demo"),
      ),
      body: buildBody.load().watch<PPage>()(context),
    );
  }

  Widget buildBody(BuildContext context) {
    return Center(
      child: InkWell(
        child: ((_) => Text(controller.text)).watch<Child>()(context),
        onTap: () {
          Navigator.of(context).push(
            MaterialPageRoute(builder: (_) => ExamplePage2()),
          );
        },
      ),
    );
  }

  Widget buildBody2(BuildContext context) {
    return ListView.separated(
      padding: EdgeInsets.all(10),
      itemBuilder: (_, index) {
        return ListTile(title: Text(controller.data[index]));
      },
      separatorBuilder: (_, index) {
        return Divider(height: 1);
      },
      itemCount: controller.data.length,
    );
  }

  @override
  ExampleController get controller => _controller;
}

class ExampleController extends IController {
  String text = "android";

  List<String> data = [];

  @override
  mount(IState state) {
    super.mount(state);
    AppProvider.expose(buildContext, this);
  }

  @override
  Future<int> load([int page]) async {
    await Future.delayed(Duration(seconds: 1), () {
      text = "flutter";
      get<PPage>().notify();
    });
    useLevel<Child>();
    return LoadState.loaded;
  }

  // @override
  // Future<int> load([int page]) async {
  //   final newData = await _loadData(page);
  //   return computeLoadingState(data, newData, page, pageRows: 50);
  // }

  Future<List<String>> _loadData(int page) async {
    await Future.delayed(Duration(seconds: 2));
    if (page > 3) return [];

    return List<String>.generate(
      50,
      (index) => ((page - 1) * 50 + index + 1).toString(),
    );
  }
}

class ExamplePage2 extends IPage {
  @override
  IState<IPage, IController> createState() => Example2State();
}

class Example2State extends IState<ExamplePage2, Example2Controller> {
  Example2Controller _controller = Example2Controller();

  @override
  Widget buildChild(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text("page 2"),),
      body: Center(
        child:
        Text("来自第一页的数据： ${AppProvider.get<ExampleController>(context).text}"),
      ),
    );
  }

  @override
  Example2Controller get controller => _controller;
}

class Example2Controller extends IController {
  @override
  Future<int> load([int page]) async {
    return LoadState.loaded;
  }
}
